use clap::Parser;
use std::io::{self, Read};
use atty::Stream;
use std::time::{Duration};

mod tcp;

use tcp::tcp_scan;
use tcp::tcp_stress;

mod http;
use http::http_stress;

mod results;


fn main() -> std::io::Result<()> {
    let args = Args::parse();
    let address = args.address;
    let timeout = Duration::from_secs(args.timeout);
    let sleep = Duration::from_millis(args.wait);
    let duration = Duration::from_secs(args.duration);
    let ports = handle_ports(args.ports.clone(), &args.range);
    let threads = args.cthreads;

    print_banner();
    
    //Reading the payload from stdin (if applicable)
    let mut payload = String::new();
    let aatty = atty::is(Stream::Stdin);

    if !aatty {
        io::stdin().read_to_string(&mut payload)?;
    } else {
        payload = "SAKAMOTO".to_string();
    }

    match args.mode.as_str() {
        "tcpstress" => {
            println!("===== TCP STRESS TEST =====");
            tcp_stress(payload, address, timeout, duration, ports, threads)?;
            return Ok(());
        }
        "tcpscan" => {
            println!("===== TCP SCAN =====");
            tcp_scan(address, timeout, sleep, ports)?;
            return Ok(());
        }
        "httpstress" => {
            println!("===== HTTP STRESS TEST =====");
            http_stress(address, duration, sleep, threads)?;
            return Ok(());
        }
        _ => {
            panic!("Mode unkown ! See the help.");
        }
    }
}

//Returns a correct list of ports, according to range.
//If range is true, we send a range of ports. Otherwise
//we send the original back
fn handle_ports(ports: Vec<u32>, range: &bool) -> Vec<u32> {
    if *range {
        if ports.iter().count() != 2 {
            panic!("A range must be given only a start and an end value.");
        }
        let ret: Vec<u32> = (ports[0]..ports[1]).collect();
        ret
    } else {
        ports
    }
}

fn print_banner() {
    let lines = [
        "    ____  ____  ____  _______   ____  ________",
        "   / __ \\/ __ \\/ __ \\/_  __/ | / / / / /_  __/",
        "  / /_/ / / / / /_/ / / / /  |/ / / / / / /  ", 
        " / ____/ /_/ / _, _/ / / / /|  / /_/ / / /    ",
        "/_/    \\____/_/ |_| /_/ /_/ |_/\\____/ /_/  ",
    ];

    for line in lines {
        println!("{}", line);
    }
    println!("");
}




///A simple TCP/HTTP port scanner / stresser.
///If stressing, you can pass a payload via a pipe.
#[derive(Debug, Parser)]
#[command(author = "Squip", version, long_about = None)]
struct Args {
    ///Mode of use : either tcpstress, tcpscan or httpstress
    mode: String,
    ///IP address or hostname to scan - or url if using http
    #[arg(short, long)]
    address: String,
    ///Timeout for each connection in seconds
    #[arg(short, long, default_value_t = 1)]
    timeout: u64,
    ///Number of milliseconds to wait in between scans or requests
    #[arg(short, long, default_value_t = 30)]
    wait: u64,
    ///Duration of tcp stress test in seconds, default 30
    #[arg(short, long, default_value_t = 30)]
    duration: u64,
    ///Ports to stress / scan, separated by commas (22,80)
    #[arg(short, long, value_parser, num_args=0.., value_delimiter=',')]
    ports: Vec<u32>,
    ///Set this flag to treat the ports as a range rather than a list
    #[arg(long, short, action, default_value_t = false)]
    range: bool,
    ///Set this flag to stress the ports instead of scanning them
    #[arg(long, short, action, default_value_t = false)]
    stress: bool,
    ///How many threads per port when stressing
    ///(Concurrent Threads)
    #[arg(short, long, default_value_t = 5)]
    cthreads: u32,
}
